<?php
/**
 * GENERIC Init
 * Contains generic Init class - with some specific loaders
 * @package WP-Favicons
 * @since 0.4.0
 * @author Edward de Leau <e@leau.net>, {@link http://edward.de.leau.net}
 * @copyright GPL 2
 */
namespace leau\co\wp_favicons;


$plugin_root = dirname( __FILE__ ) . '/../plugins/';
$all_plugins = Config::GetRegisteredPlugins();
foreach ($all_plugins as $module_key => $module_value)
{
	foreach ($module_value as $plugin_key => $plugin_value)
	{
		require_once $plugin_root . $module_key . '/' . $plugin_key . '.php';
	}
}

if (!class_exists("Init"))
{
	/**
	 * Init
	 * Generic Init class that is reusable over multiple plugins contains NO specific
	 * entries for this plugin
	 * @author Edward de Leau <e@leau.net>, http://edward.de.leau.net
	 * @since 0.4.0
	 */
	class Init
	{
		public $pluginObjects = array();

		/**
		 * constructor
		 * Initializes the plugin by registering the settings (1 array),
		 * loads the administration modules (which load the plugins) and
		 * adds the admin pages, only called on admin area
		 * @author Edward de Leau <e@leau.net>, http://edward.de.leau.net
	 	 * @since 0.4.0
		 * @uses add_action {@link http://codex.wordpress.org/Function_Reference/add_action}
		 * @uses admin_init {@link http://codex.wordpress.org/Plugin_API/Action_Reference/admin_init}
		 * @uses admin_menu {@link http://codex.wordpress.org/Plugin_API/Action_Reference/admin_menu}
		 */
		public function __construct()
		{
			/*
			 * instantiate all registered plugins dynamically
			 * and add the admin panels
			 */

			$all_plugins = Config::GetRegisteredPlugins();
			$n=0;
			foreach ($all_plugins as $module_key => $module_value)
			{
				foreach ($module_value as $plugin_key => $plugin_value)
				{
					$class_name = 'leau\\co\\wp_favicons\\' .
						$module_key . "_" . $plugin_key;
					//$class_name = $module_key . "_" . $plugin_key;

					$this->pluginObjects[$n] = new $class_name;
					$this->pluginObjects[$n]->Activate($module_key,$plugin_key);
					add_action(Config::GetPluginSlug() . $module_key .
						'_plugins', array($this->pluginObjects[$n],
						'AddAdminOption'), 1, 10);
					$n++;
				}
			}

			/*
			 * after first time activation write a finalizer
			 * to prevent multiple lookups for the existence of option keys
			 * thereafter, when a new plugin gets added this boolean should
			 * be cleared to run it again.
			 */

			if (false == Config::GetOptionsArrayValue('FINAL'))
			{
				Database::Install();
				Config::UpdateOptions(array('FINAL' => true));
			}

			/*
			 * for admin panel:
			 * - register the settings
			 * - load all modules (subpages)
			 * - and then let plugins fill the module's content
			 */

			if (is_admin())
			{
				add_action('admin_init', array($this,'RegisterSettings'), 1, 1 );
				new Modules;
				add_action('admin_menu', array( $this, 'AddAdminPages' ), 1, 1 );
			}
		}

		/**
		 * Registers the settings
		 * @author Edward de Leau <e@leau.net>, http://edward.de.leau.net
	 	 * @since 0.4.0
		 * @uses register_settings {@link http://codex.wordpress.org/Function_Reference/register_setting}
		 */
		function RegisterSettings()
		{
			register_setting(Config::GetSettingsGroupName(),
				Config::GetOptionsName(),
				array( &$this, 'MergeOptions' ));
		}

		/**
		 * Merges the option arrays (Validation function)
		 * We want 1 options array managed on multiple pages so we need to merge it after save
		 * @author Edward de Leau <e@leau.net>, http://edward.de.leau.net
	 	 * @since 0.4.0
	 	 * @uses add_settings_error {@link http://codex.wordpress.org/Function_Reference/add_settings_error}
		 * @param array $page_options the array filled within the specific admin form
		 * @return array $validated_options the merged array containing all the plugins options
		 */
		function MergeOptions( $page_options ) {
			/* start with an empty array to prevent evil arrays */
			$validated_options = array();

			/*
			 * plugins will validate their fields via the validatefields filter:
			 * - they check wether the page_options are valid for them
			 * - and if so add the validated fields to the validated options
			 * - if an error is encountered they will set a settings error
			 * So: without validation NO values of the plugin will be saved!
			 */
			$plugin_validated_options =
				apply_filters( Config::GetPluginSlug() . 'validatefields',
					$validated_options, $page_options );

			if (is_array($plugin_validated_options))
			{
				$validated_options = $plugin_validated_options;
			}

			/*
			 * While the plugins now have validated their fields and
			 * added it to the validated array, it could well so that we
			 * have additional options that are stored outide of plugins
			 * so we need to merge these into the final array
			 */
			$merged_options = array();
        	Config::SetOptionsFromArray(get_option(Config::GetOptionsName()));
        	if ( !is_array(Config::GetOptionsAsArray() ) ||
        		 !is_array( $validated_options ))
        	{
        		add_settings_error(
					Config::GetPageTitle(),
					'settings_not_merged',
					__('Could not merge existing options with page options'),
					'error'
				);
        	}
        	else
        	{
        		$merged_options = array_merge( Config::GetOptionsAsArray(),
        			$validated_options);
        	}

        	/* return the merged validated array */
        	return $merged_options;
    	}

		/**
		 * Adds the main entry admin root and via action the admin pages added by plugins
		 * @author Edward de Leau <e@leau.net>, http://edward.de.leau.net
	 	 * @since 0.4.0
		 * @uses add_menu_page {@link http://codex.wordpress.org/Function_Reference/add_menu_page}
		 * @uses do_action {@link http://codex.wordpress.org/Function_Reference/do_action}
		 */
		function AddAdminPages()
		{
			if ( function_exists( 'add_menu_page' ) )
				add_menu_page( Config::GetPageTitle(),
					Config::GetMenuTitle(),
					Config::GetCapability(),
					Config::GetMenuSlug(),
					array( $this, 'MainPage' ),
					Config::GetMenuIcon()
				);
			/*
			 * modules === subpages can be added via plugins via the following action
			 * meaning: if you want to add a new module just add an action to this
			 * to create it (and make sure you add the new module to the config class)
			 */
			do_action(Config::GetPluginSlug() . 'modules');
		}

		/**
		 * Splash page Info
		 */
		function MainPage() {
			echo Config::GetMainPage();
		}

	}
}
new Init;